@@ -218,7 +218,7 @@ class TestCoveragePlugin(Plugin):
         if self.record:
             self.db.upgrade()
 
-        if not (self.discover or self.report):
+        if not (self.discover or self.report_coverage):
             return
 
         s = time.time()
@@ -273,34 +273,71 @@ class TestCoveragePlugin(Plugin):
             if not self.discover:
                 continue
 
-        self.logger.info("Parsed diff in %.2fs", time.time() - s)
+        self.logger.info("Parsed diff in %.2fs as %d file(s)", time.time() - s, len(diff))
 
         if self.discover:
             self.logger.info("Finding coverage for %d file(s)", len(files))
 
             for filename, linenos in diff.iteritems():
-                # TODO: for this to be useful we need to eliminate tests
-                if not self.db.has_test_coverage(filename):
-                    if self.skip_missing:
-                        self.logger.warning('%s has no test coverage recorded', filename)
-                        continue
-                    raise AssertionError("Missing test coverage for %s" % filename)
-
-                # record
-                for test in self.db.get_test_coverage(filename, linenos):
-                    pending_funcs.add(test)
+                test_coverage = self.db.get_test_coverage(filename, linenos)
+                if not test_coverage:
+                    # check if we have any coverage recorded
+                    if not self.db.has_test_coverage(filename):
+                        if self.skip_missing:
+                            self.logger.warning('%s has no test coverage recorded', filename)
+                            continue
+                        raise AssertionError("Missing test coverage for %s" % filename)
+                else:
+                    for test in test_coverage:
+                        pending_funcs.add(test)
 
             self.logger.info("Determined available coverage in %.2fs with %d test(s)", time.time() - s, len(pending_funcs))
 
+        if self.report_coverage and not self.record:
+            # If we're recording coverage we need to ensure it gets reset
+            self.coverage = coverage()
+            self.coverage.start()
+
     def report(self, stream):
         if not self.report_coverage:
             return
 
+        cov_data = self.cov_data
+        diff_data = self.diff_data
+
+        if not self.record:
+            cov = self.coverage
+            cov.stop()
+
+            # initialize reporter
+            rep = Reporter(cov)
+
+            # process all files
+            rep.find_code_units(None, cov.config)
+
+            for cu in rep.code_units:
+                # if sys.modules[test_.__module__].__file__ == cu.filename:
+                #     continue
+                filename = cu.name + '.py'
+                try:
+                    # TODO: this CANT work in all cases, must be a better way
+                    analysis = rep.coverage._analyze(cu)
+                    linenos = analysis.statements
+                    if self.report_coverage:
+                        diff = diff_data[filename]
+                        cov_linenos = [l for l in linenos if l in diff]
+                        if cov_linenos:
+                            cov_data[filename].update(cov_linenos)
+                except KeyboardInterrupt:                       # pragma: no cover
+                    raise
+                except:
+                    traceback.print_exc()
+
         covered = 0
         total = 0
         missing = defaultdict(set)
-        for filename, linenos in self.diff_data.iteritems():
-            covered_linenos = self.cov_data[filename]
+        for filename, linenos in diff_data.iteritems():
+            covered_linenos = cov_data[filename]
 
             total += len(linenos)
             covered += len(covered_linenos)
@@ -353,14 +390,15 @@ class TestCoveragePlugin(Plugin):
         return False
 
     def startTest(self, test):
-        if not (self.report_coverage or self.record):
+        if not self.record:
             return
 
-        self.coverage = coverage(include='disqus/*')
+        # If we're recording coverage we need to ensure it gets reset
+        self.coverage = coverage()
         self.coverage.start()
 
     def stopTest(self, test):
-        if not (self.report_coverage or self.record):
+        if not self.record:
             return
 
         cov = self.coverage
